#  Copyright 2022 Terra Enabling Developers Limited
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
__all__ = ["TerraException", "NoDtypeException", "NoBodyException", "NoUserInfoException", "NoClientAvailable"]


class TerraException(Exception):
    """Base class for all exceptions raised by this library"""


class NoClientAvailable(TerraException):
    """
    Exception raised when a TerraUser object has no Terra client set,
    but a method requiring it is called.
    """


class NoBodyException(TerraException):
    """
    Exception raised when a TerraApi object has no body
    """


class NoUserInfoException(TerraException):
    """
    Exception raised when a UserInfoDict is None
    """


class NoDtypeException(TerraException):
    """
    Exception raised when a TerraPi object has no dtype
    """
